import { AfterViewInit, Component, EventEmitter, Input, NgModule, OnInit, Output, Renderer2, ViewChild } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';

import { DataTable, DataTableModule } from 'primeng/components/datatable/datatable';
import { DropdownModule } from 'primeng/components/dropdown/dropdown';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';
import { SharedModule } from 'primeng/components/common/shared';

import { ProfileViewerModule } from '../profileViewer/profileViewer';
import { LoadingBannerModule } from '../loadingBanner/loadingBanner';
import { SliderModule } from '../slider/slider';

import { fadeInOutAnimation } from '../kernel/animations/fade.animation';

import { GlxSharedModule } from '../kernel/shared.module';

/**
 * Фильтр по сотрудникам
 */
class Filter {
    fullname: string;
    job_title: string;
    department: string;
    global: string;

    constructor() {

    }
}

/**
 * Поиск сотрудников с детальным просмотром профиля
 */
@Component({
    selector: 'glx-staff-search',
    styleUrls: ['./staffSearch.scss'],
    templateUrl: './staffSearch.html',
    animations: [fadeInOutAnimation],
})

export class StaffSearch implements OnInit, AfterViewInit {
    /**
     * Данные для отображения в таблице
     */
    @Input() data: any = {};
    /**
     * Справочники для фильтров
     * @type {{}}
     */
    @Input() catalog: any = {};
    /**
     * Профиль для детального просмотра
     * @type {{}}
     */
    @Input() profile: any = {};
    /**
     * Открыт ли фильтр
     * @type {boolean}
     */
    @Input() filterShow: boolean = false;
    /**
     * Показать индикатор загрузки данных
     * @type {boolean}
     */
    @Input() loading: boolean = false;
    /**
     * Признак того, что компонент открыт в слайдере
     * @type {boolean}
     */
    @Input() slider: boolean = false;
    /**
     * Изображение профиля по-умолчанию
     * @type {string}
     */
    @Input() defaultImage: string = '';
    /**
     * Задержка поиска для общего фильтра
     * @type {number}
     */
    @Input() filterDelay: number = 1000;
    /**
     * Фильтр
     * @type {Filter}
     */
    @Input() filter: Filter = new Filter();
    /**
     * Надпись на глобальном фильтре
     * @type {string}
     */
    @Input() globalFilterPlaceholder: string = 'Поиск...';
    /**
     * Событие фильтрации. Необходимо получить новые данные сервера
     * @type {EventEmitter<any>}
     */
    @Output() onFilter: EventEmitter<any> = new EventEmitter();
    /**
     * Событий просмотра профиля. Необходимо получить профиль пользователя с сервера
     * @type {EventEmitter<any>}
     */
    @Output() onViewProfile: EventEmitter<any> = new EventEmitter();
    @Output() onViewManager: EventEmitter<any> = new EventEmitter();
    @Output() exportAction: EventEmitter<any> = new EventEmitter();

    /**
     * Поле для предфильтрации
     */
    @Input() filterField: any;
    /**
     * Значение поля для предфильтрации
     */
    @Input() filterValue: any;
    /**
     * Префикс пути для изображений
     */
    @Input() imagePrefix: string;

    /**
     * PrimeNG DataTable
     */
    @ViewChild('dt') datatable: DataTable;
    /**
     * PrimeNg Global Search Field
     */
    @ViewChild('gb') gb: any;
    /**
     * Filter Department Field
     */
    @ViewChild('filterDepartment') filterDepartment: any;

    /**
     * Флаг открытого слайдера с профилем
     * @type {boolean}
     */
    showSide: boolean = false;  // показать профиль
    /**
     * ID профиля
     */
    profile_id: number;

    /**
     * Timeout экземпляр фильтрации
     */
    filterTimeout: any;
    /**
     * Первое срабатывание фильтрации
     * @type {boolean}
     */
    firstFilter: boolean = true;

    /**
     * Конструктор
     * @param {Router} router
     * @param {ActivatedRoute} route
     * @param {Renderer2} renderer
     */
    constructor(private router: Router,
                private route: ActivatedRoute,
                private renderer: Renderer2) {
    }

    /**
     * Проверка префильтров
     */
    ngOnInit() {
        this.route.paramMap.subscribe(
            params => {
                if (!this.slider) {
                    this.filterField = params.get('field');
                    this.filterValue = params.get('value');
                    if (!this.firstFilter) {
                        this.checkPreFilter();
                        this.datatable._filter();
                        if (this.showSide) {
                            this.toggleSide();
                        }
                    }
                }
            }
        );
    }

    /**
     * Получить путь к фото
     * @param photo
     * @returns {any}
     */
    getPhoto(photo) {
        if (!photo) {
            return null;
        }
        return this.imagePrefix + photo;
    }

    /**
     * Выключить поле если по нему есть префильтрация
     * @param field
     * @returns {boolean}
     */
    ifDisabled(field) {
        return !!(this.filterField && this.filterValue && this.filterField === field);
    }

    /**
     * Проверка префильтра
     */
    checkPreFilter() {
        if (this.route.snapshot.paramMap.get('field') && this.route.snapshot.paramMap.get('value') && this.firstFilter) {
            this.filterField = this.route.snapshot.paramMap.get('field');
            this.filterValue = this.route.snapshot.paramMap.get('value');
        }

        if (this.filterField && this.filterValue) {
            this.filter[this.filterField] = this.filterValue;
            if (this.filterValue === parseInt(this.filterValue, 10)) {
                this.datatable.filters[this.filterField] = {
                    value: parseInt(this.filterValue, 10),
                    matchMode: 'any'
                };
            } else {
                this.datatable.filters[this.filterField] = {
                    value: this.filterValue,
                    matchMode: 'any'
                };
            }
        }
    }

    /**
     * Обработка нажатия клавиши вниз в фильре подразделений после поиска в компоненте
     */
    ngAfterViewInit() {
        this.renderer.listen(this.filterDepartment.filterViewChild.nativeElement, 'keyup', (event: KeyboardEvent) => {
            const key = event.which || event.keyCode;

            if (key === 13 || key === 40) {
                this.filterDepartment.applyFocus();
                /**
                 * из кода primeng/components/dropdown/dropdown.ts строка 419
                 * обработка нажатия клавиши вниз, чтобы после фильтрации выбрать первый отфильтрованнный элемент
                 */
                const selectedItemIndex = this.filterDepartment.selectedOption ? this.filterDepartment.findOptionIndex(
                    this.filterDepartment.selectedOption.value,
                    this.filterDepartment.optionsToDisplay
                ) : -1;
                const nextItemIndex = selectedItemIndex + 1;
                if (nextItemIndex !== (this.filterDepartment.optionsToDisplay.length)) {
                    this.filterDepartment.selectedOption = this.filterDepartment.optionsToDisplay[nextItemIndex];
                    this.filterDepartment.selectedOptionUpdated = true;
                    this.filterDepartment.selectItem(event, this.filterDepartment.selectedOption);
                }
            }
        });
    }

    /**
     * Скрыть/показать профиль пользователя
     */
    toggleSide() {
        this.showSide = !this.showSide;
    }

    /**
     * Очистить фильтр
     */
    clearFilter() {
        if (this.filterField || this.filterValue) {
            this.filterField = null;
            this.filterValue = null;

            if (!this.slider) {
                this.router.navigate(['/search/staff']);
            }
        }

        this.filter = new Filter();
        if (this.datatable.globalFilter && this.datatable.globalFilter.value) {
            this.datatable.globalFilter.value = '';
        }
        this.datatable.reset();
        this.datatable.sortField = 'fullname';
        this.datatable._filter();
    }

    /**
     * Скрыть / показать фильтры
     */
    toggleFilter() {
        this.filterShow = !this.filterShow;
    }

    /**
     * Установить заголовок слайдера с профилем
     * @returns {string}
     */
    setProfileHeader() {
        if (this.profile && this.profile.role && this.profile.role.length > 0) {
            return this.profile.user.fullname;
        }
        return '';
    }

    /**
     * Применение глобального фильтра
     * @param gbt
     */
    globalFilterApply(gbt) {
        if (this.filterTimeout) {
            clearTimeout(this.filterTimeout);
        }
        if (gbt && gbt.value && gbt.value.length > 2) {
            this.filterTimeout = setTimeout(() => {
                this.datatable.globalFilter = gbt;
                this.datatable._filter();
                this.filterTimeout = null;
            }, this.filterDelay);
        }
        if (gbt && gbt.value === '') {
            this.datatable.globalFilter.value = '';
            this.datatable._filter();
        }
    }

    /**
     * Установка и применение фильтра
     * @param name
     * @param value
     */
    setFilter(name, value) {
        switch (name) {
            case 'fullname':
                if (this.filterTimeout) {
                    clearTimeout(this.filterTimeout);
                }
                if (value && value.length > 2) {
                    this.filterTimeout = setTimeout(() => {
                        this.datatable.filter(value, name, 'any');
                        this.filterTimeout = null;
                    }, this.filterDelay);
                }
                if (value === '') {
                    this.datatable.filter(value, name, 'any');
                }
                break;
            default:
                this.datatable.filter(value, name, 'any');
        }

    }

    /**
     * Устанавилвает фильтр из контекста (при клике на значение в таблице)
     * @param name string Название поля для фильтрации
     * @param value string Значение поля
     */
    setContextFilter(name, value) {
        this.filter = new Filter();
        this.datatable.filters = {};
        this.datatable.filteredValue = null;
        this.datatable.sortField = 'fullname';
        this.filter[name] = value;
        this.setFilter(name, value);
    }

    /**
     * Событие для ленивой загрузки. Вызывается при инициализации и фильтрации
     * @param event
     */
    loadItemsLazy(event: any) {
        if (this.firstFilter) {
            this.checkPreFilter();
            this.firstFilter = false;
        }

        this.onFilter.emit(event);
    }

    /**
     * Собатие показа профиля в слайдере
     * @param item
     */
    showProfile(item ?) {
        if (item) {
            this.profile_id = item.id;
        }

        if (this.profile_id > 0) {
            this.onViewProfile.emit(this.profile_id);
            this.toggleSide();
        } else {
            console.error('Ошибка: Не получен ID профиля');
        }
    }

    /**
     * Двойной щелчок по строке открывает профиль
     * @param event
     */
    onDblRowclick() {
        this.showProfile();
    }

    /**
     * Одиночный щелчок по строке устанавливает ID профиля для возможного открытия
     * @param event
     */
    onRowClick(event) {
        this.profile_id = event.data.id;
    }

    /**
     * Установка подсказки в фильтре подразделений
     * @param item
     * @returns {any}
     */
    getLabel(item) {
        if (item && item.tree && item.tree.length > 0) {
            let deps = '';
            for (let i = 0; i < item.tree.length; i++) {
                deps += '<div class="dep" style="padding-left: ' + (i * 8) + 'px">' + item.tree[i].title + '</div>';
            }
            return `
            <div class="tooltip-custom-content">
               ` + deps + `
            </div>
        `;
        } else {
            return '';
        }
    }

    /**
     * Получить название последнего подразделения в дереве где работает сотрудник
     * @param deps
     */
    getDepartmentTitle(deps) {
        let dep = deps;
        if (dep.child) {
            while (dep.child) {
                dep = dep.child;
            }
        }
        return dep.title;
    }

    /**
     * Получить ID последнего подраделения где работает сотрудник
     * @param deps
     */
    getDepartmentId(deps) {
        let dep = deps;
        if (dep.child) {
            while (dep.child) {
                dep = dep.child;
            }
        }
        return dep.id;
    }

    managerViewHandler(e) {
        this.onViewManager.emit({id: e.id});
    }

    export() {
        this.exportAction.emit({filter: this.datatable.createLazyLoadMetadata()});
    }
}

@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        RouterModule,
        SharedModule,
        GlxSharedModule,
        DataTableModule,
        DropdownModule,
        TooltipModule,
        ProfileViewerModule,
        LoadingBannerModule,
        SliderModule
    ],
    exports: [StaffSearch],
    declarations: [StaffSearch]
})

export class StaffSearchModule {
}
